/*******************************************************************************
*  skScanSW.cpp - スイッチのチャタリング除去読込みを行う関数用ライブラリ       *
*                                                                              *
*    SW_Init - チャタリング除去読込みをする為の初期化を行う処理                *
*    ScanSW_Interrupt - 定期的にスイッチの状態読込みを行う割込みの処理         *
*    SW_read - スイッチのON/OFFを調べる時に使用する処理                        *
*                                                                              *
*    note : Arduino Duemilanove 328 / ArduinoIDE 1.0.1 で試験開発を行ってます。*
*                                                                              *
* ============================================================================ *
*   VERSION  DATE        BY             CHANGE/COMMENT                         *
* ---------------------------------------------------------------------------- *
*   1.00     2014-04-22  きむ茶工房     Create                                 *
*******************************************************************************/
#include <MsTimer2.h>              // タイマー割り込みを利用する為に必要なヘッダファイル
#include "ScanSW.h"

byte SW_mask ;
int  SW_cnt ;
volatile byte SW_data[8][SW_COUNTER+2] ;

/*******************************************************************************
*  SW_Init(ms,inf)                                                             *
*    スイッチのチャタリング除去読込みをする為の初期化を行う処理                *
*    MsTimer2の機能を利用しています。                                          *
*                                                                              *
*    ms : 割り込み関数を呼び出すタイミングの時間(ms)を指定する                 *
*    inf: 接続している端子を指定する(但しDIGITAL2～7までに繋ぐ事)              *
*         DIGITAL2と7に接続しているなら"B10000100"と指定                       *
*******************************************************************************/
void SW_Init(int ms,byte inf)
{
     int i ;
     
     // スイッチのデータ変数を初期化する
     SW_cnt = SW_COUNTER + 1 ;
     for (i=0 ; i<8 ;i++) {
          SW_data[i][0]      = 0 ;      // SWの状態をリセット
          SW_data[i][SW_cnt] = 0 ;      // 読込み回数のカウントをリセット
     }
     // ポートＤレジスタの入出力を設定する
     SW_mask = inf ;
     DDRD = DDRD & ~SW_mask ;                // 指定ピンを入力にする(０で入力)
     // タイマー割り込みの設定をする
     MsTimer2::set(ms, ScanSW_Interrupt) ;   // 指定ms毎に割込み関数を呼び出す様に設定
     MsTimer2::start();                      // タイマー割り込み開始
}
/*******************************************************************************
*  SW_read(value)                                                              *
*    スイッチのON/OFFを調べる時に使用します。                                  *
*    この処理は"ScanSW.h"にdefine記述されています。                            *
*                                                                              *
*    value : 調べたい端子の番号を指定する                                      *
*            DIGITAL2を見たいなら"SW_read(2)"とする                            *
*******************************************************************************/


/*******************************************************************************
*  ScanSW_Interrupt()                                                          *
*    スイッチの状態を読み込みする処理                                          *
*    この関数は定期的にMsTimer2から起動される割り込みの処理です。              *
*******************************************************************************/
void ScanSW_Interrupt()
{
     int i ;

     // DIGITAL2-7の分だけ処理を繰り返す
     for (i=2 ; i<8 ;i++) {
          // スイッチが繋がっている端子のみスキャンする
          if (bitRead(SW_mask,i) == HIGH) {
               SW_data[i][SW_cnt]++ ;                              // 読込み回数のカウントを行う
               SW_data[i][SW_data[i][SW_cnt]] = bitRead(PIND,i) ;  // SWの状態を読込む
               if (SW_data[i][SW_cnt] > 1) {
                    if (SW_data[i][1] != SW_data[i][SW_data[i][SW_cnt]]) {
                         // １回目の読込み値とSWの状態が異なる(2回目以降を1回目の値とする)
                         SW_data[i][1]      = SW_data[i][SW_data[i][SW_cnt]] ;
                         SW_data[i][SW_cnt] = 1 ;                  // 1回目のカウント値
                    }
               }
               if (SW_data[i][SW_cnt] >= SW_COUNTER) {
                    // 読込み値が全て同じであるならその値をSWの状態とする
                    SW_data[i][0]      = SW_data[i][1] ;
                    SW_data[i][SW_cnt] = 0 ;                       // カウント値をリセット
               }
          }
     }
}
